package com.qy.worksheetsystem.service.impl;


import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.kingdee.shr.api.OSFWSClient;
import com.qy.worksheetsystem.model.entity.JobBooking;
import com.qy.worksheetsystem.model.entity.JobBookingEntry;
import com.qy.worksheetsystem.model.mapper.PersonWorkMapper;

import com.qy.worksheetsystem.model.mapper.ProjectMapper;
import com.qy.worksheetsystem.service.PersonService;
import com.qy.worksheetsystem.service.PersonWorkService;
import com.qy.worksheetsystem.util.SnowflakeUtils;
import com.qy.worksheetsystem.vo.WorkHoursReport;
import com.qy.worksheetsystem.vo.WorkHoursReportEntry;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @BelongsProject: workSheetSystem
 * @BelongsPackage: com.qy.worksheetsystem.service.impl
 * @Author: lhbj
 * @CreateTime: 2024-12-03  15:18
 * @Description: TODO
 * @Version: 1.0
 */
@Slf4j
@Service
public class PersonWorkServiceImpl implements PersonWorkService {
    private static final BigDecimal h8 = new BigDecimal(8);

    @Resource
    private PersonWorkMapper personWorkMapper;
    @Resource
    private PersonService personService;
    @Resource
    private ProjectMapper projectMapper;

    @Value("${HR.ais.url}")
    private String url;
    @Value("${HR.ais.serviceName}")
    private String serviceName;

    @Override
    @Transactional
    public Map<String,Object> insertWork(Map<String,Object> param){
        List<Map<String, Object>> billList = Lists.newArrayList();
        List<Map<String, Object>> workList = Lists.newArrayList();
        int begin = (int) param.get("begin");
        int end = (int) param.get("end");
        String yearMonth = (String) param.get("yearMonth");
        WorkHoursReport jobWork = (WorkHoursReport) param.get("jobWork");
        Map<String, String> person = (Map<String, String>) param.get("person");
        Date beginDate = (Date) param.get("beginDate");
        Date endDate = (Date) param.get("endDate");
        Map<String, Map<String, Object>> availableMap = (Map<String, Map<String, Object>>) param.get("availableMap");
        Map<String, Object> reMap = Maps.newHashMap();
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");


            Integer Hours = 0;
            for (WorkHoursReportEntry jobWorkEntry : jobWork.getEntries()) {
                String workingHours = jobWorkEntry.getWorkingHours();
                BigDecimal Hour = new BigDecimal(workingHours);
                Hours += Hour.intValue();
                String projectId = jobWorkEntry.getProjectId();

                String number = SnowflakeUtils.getInstance().getSnowflake().nextIdStr();

                Map<String, Object> billMap = new HashMap<>();

                String billID = personWorkMapper.selectBillKey(new Long(number));
                billMap.put("id", billID);
                billMap.put("affiliatedProject", projectId);
                billMap.put("fillingStartDate", jobWork.getBeginTime());
                billMap.put("fillingEndDate", jobWork.getEndTime());
                billMap.put("adminOrg", person.get("adminOrgId"));
                billMap.put("hrOrgUnit", person.get("hrOrgUnitID"));
                billMap.put("billState", "0");
                billMap.put("number", number);
                billMap.put("CU", person.get("controlUnitID"));
                Date createTime = new Date();
                billMap.put("creator", person.get("userID"));
                billMap.put("lastUpdateUser", person.get("userID"));
                for (int i = begin; i <= end; i++) {
                    Map<String, Object> objectMap = Maps.newHashMap();
                    String iday = "01";
                    if (i < 10) {
                        iday = "0" + i;
                    } else {
                        iday = i + "";
                    }
                    String reportDate = yearMonth + iday;
                    Map<String, Object> avMap = availableMap.get(reportDate);
                    BigDecimal availableWorkHous = (BigDecimal) avMap.get("availableWorkHours");
                    BigDecimal workHours = (BigDecimal) avMap.get("workHours");
                    objectMap.put("parent", billID);
                    objectMap.put("employee", person.get("id"));
                    objectMap.put("clockTime", reportDate);
                    objectMap.put("clockLocation", "移动端");
                    objectMap.put("data", reportDate);
                    objectMap.put("WorkingHours", Hour);
                    objectMap.put("notes", "");
                    objectMap.put("seq", i);
                    workList.add(objectMap);

                }
                billList.add(billMap);
            }

            int bInt = personWorkMapper.insertBill(billList);
            int wInt = personWorkMapper.insertWork(workList);
            reMap.put("billList",billList);
            reMap.put("workList",workList);
            reMap.put("code", (billList.size()+workList.size())==(bInt+wInt));
            return reMap;
        }catch (Exception e){
            log.error(e.getMessage(),e);
            for(Map<String,Object> bill:billList) {
                this.deleteWork(person.get("personNumber"), (String) bill.get("id"));
            }
            reMap.put("errMsg", e.getMessage());
            reMap.put("code", false);
        }
        return reMap;
    }

    @Override
    public List<Map<String, Object>> getAvailableDateInfoById(String number,String id, String startDate, String endDate) {
        Map<String,String> person = personService.getPersonByJobNo(number);
        List<Map<String,Object>> list = personWorkMapper.getAvailableDateInfoById(person.get("id"),id,startDate,endDate);
        for(Map<String,Object> map : list){

            BigDecimal workHours = (BigDecimal) map.get("workHours");
            if(null!=workHours) {
                map.put("availableWorkHours", h8.subtract(workHours));
            }else{
                map.put("workHours", BigDecimal.ZERO);
                map.put("availableWorkHours", h8);
            }
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> getAvailableDateInfo(String number, String startDate, String endDate) {
        Map<String,String> person = personService.getPersonByJobNo(number);
        List<Map<String,Object>> list = personWorkMapper.getAvailableDateInfo(person.get("id"),startDate,endDate);
        for(Map<String,Object> map : list){

            BigDecimal workHours = (BigDecimal) map.get("workHours");
            if(null!=workHours) {
                map.put("availableWorkHours", h8.subtract(workHours));
            }else{
                map.put("workHours", BigDecimal.ZERO);
                map.put("availableWorkHours", h8);
            }
        }
        return list;
    }

    @Override
    public List<Map<String,Object>> getPersonWorkList(String number,String beginTime, String endTime, Boolean state) {
        List<Map<String,Object>> list = personWorkMapper.getPersonWorkList(number,beginTime,endTime,state);
        return list;
    }
    @Override
    public   List<JobBooking> getPersonWorkListV2(String number, String beginTime, String endTime, Boolean state) {
        List<JobBooking> list = personWorkMapper.getPersonWorkListV2(number,beginTime,endTime,state);
        for(JobBooking job : list){
            BigDecimal workingHours = BigDecimal.ZERO;
            BigDecimal hours = BigDecimal.ZERO;
            for(JobBookingEntry jobEntry : (List<JobBookingEntry>)job.get("entries")){
                BigDecimal work = new BigDecimal(jobEntry.getWorkingHours());
                workingHours=workingHours.add(work);
                hours=work;
            }
            job.put("WorkingHours",hours.toPlainString());
        }
        return list;
    }
    @Override
    public List<Map<String,Object>> getPersonWorkSumInfo(String number,String beginTime,String endTime,Boolean state){
        List<Map<String,Object>> list =personWorkMapper.getPersonWorkSumInfo(number,beginTime,endTime,state);
        return list;
    }
    @Override
    public JobBooking getPersonWorkInfoById(String number, String id){
        JobBooking e =personWorkMapper.getPersonWorkInfoById( number, id);
        BigDecimal workingHours = BigDecimal.ZERO;
        for(JobBookingEntry jobEntry : (List<JobBookingEntry>)e.get("entries")){

            workingHours=workingHours.add(new BigDecimal(jobEntry.getWorkingHours()));
        }
        e.put("WorkingHours",workingHours.toPlainString());
        return e;
    }
    @Override
    @Transactional
    public int deleteWork(String number, String id){
        int x =personWorkMapper.deleteWorkEntry(id);
        int i =personWorkMapper.deleteWork(id);
        return i;
    }
    @Override
    @Transactional
    public int submitWork(String number, String id){
        OSFWSClient client = new OSFWSClient();
        Map<String,Object> map = Maps.newHashMap();
        map.put("billId",id);//  单据id
        map.put("status","1");//  单据id
        String str = client.proceedOSF(url,serviceName,map);
        if(StrUtil.isNotBlank(str)){
            JSONObject j = JSONUtil.parseObj(str);
            String code = j.getStr("code");
            if("200".equals(code)){
                return 1;
            }
        }
        return 0;
    }
    @Override
    @Transactional
    public int revocaWork(String number, String id){
        OSFWSClient client = new OSFWSClient();
        Map<String,Object> map = Maps.newHashMap();
        map.put("billId",id);//  单据id
        map.put("status","2");//  单据id
        String str = client.proceedOSF(url,serviceName,map);
        if(StrUtil.isNotBlank(str)){
            JSONObject j = JSONUtil.parseObj(str);
            String code = j.getStr("code");
            if("200".equals(code)){
                return 1;
            }
        }
        return 0;
    }
}
